// Copyright (c) 2004-2016  Pavel Rychly, Milos Jakubicek

#include <finlib/fromtof.hh>
#include "config.hh"
#include "subcorp.hh"
#include <iostream>
#include <cstdlib>

#if defined MANATEE_HAVE_EXT_HASH_MAP
#include <ext/hash_map>
#    if defined MANATEE_HAVE_GNU_CXX_NS
         namespace std {using namespace __gnu_cxx;}
#    endif
#else
#include <hash_map>
#endif

using namespace std;

int main (int argc, char **argv) 
{
    if (argc < 3) {
        cerr << "usage: mknorms CORPUS STRUCT [NORM_STRUCTATTR SUBCORPFILE]\n"
             << "(use - as NORM_STRUCTATTR to use structure size)\n"
             << "example: mknorms mycorp doc wordcount\n";
        return 2;
    }
    try {
        Corpus corp (argv[1]);
        Structure *st = corp.get_struct (argv[2]);
        if (CorpInfo::str2bool(st->conf->opts["NESTED"])) {
            cerr << "Norm structure must not be NESTED, exiting.\n";
            return 3;
        }
        hash_map<uint64_t,uint64_t> norms;
        string normattrname;
        
        // init norms hash
        cerr << "preparing norm counts ..." << endl;
        if (argc >= 4 && argv[3][0] != '-') {
            normattrname = argv[3];
            PosAttr *na = st->get_attr (normattrname);
            TextIterator *it = na->textat (0);
            for (NumOfPos i = 0; i < st->size(); i++)
                norms[st->rng->beg_at(i)] = atoi (it->next());
            delete it;
        } else {
            for (NumOfPos i = 0; i < st->size(); i++) {
                Position beg = st->rng->beg_at(i);
                norms[beg] = st->rng->end_at(i) - beg;
            }
        }
        SubCorpus *subc = NULL;
        string path = st->conf->opts["PATH"];
        if (argc == 5) {
            subc = new SubCorpus (&corp, argv[4]);
            path = string (argv[4], strlen (argv[4]) - 4) + st->name + '.';
        }

        // save norms
        CorpInfo::VSC::const_iterator sattr = st->conf->attrs.begin();
        for (; sattr != st->conf->attrs.end(); ++sattr) {
            const string &aname = sattr->first;
            if (aname == normattrname)
                continue;
            cerr << "saving norms for " << aname << " ..." << endl;
            ToFile<int64_t> normf (path + aname + ".norm");
            PosAttr *at = st->get_attr (aname);
            int id, id_range = at->id_range();
            for (id = 0; id < id_range; id++) {
                NumOfPos ntotal = 0;
                FastStream *pos = at->id2poss (id);
                RangeStream *rs = st->rng->part (pos);
                if (subc)
                    rs = subc->filter_query (rs);
                while (!rs->end()) {
                    ntotal += norms[rs->peek_beg()];
                    rs->next();
                }
                delete pos;
                normf.put (ntotal);
            }
        }
    } catch (exception &e) {
        cerr << "mknorms error: " << e.what() << '\n';
        return 1;
    }
    return 0;
}

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
